/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * COPYING for more details. */
/*
 * ShellBlade - The Shellcode Warrior's Blade
 * ShellBlade.net
 *
 * Prints shellcode in several formats, warns of nullbyte
 * presence and executes shellcode.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>

#define VERSION	"1.0"
#define MAXBYTES	15

const char help[] =
"Usage: shellblade <mode> <object file>\n\n"
 "MODES\n"
"-e\t\t\t\texecutes shellcode\n"
"-p\t\t\t\tdumps the shellcode in plain format\n"
"-c\t\t\t\tdumps the shellcode in C syntax format\n"
"-a\t\t\t\tdumps the shellcode in ASM syntax format\n"
"\n"
"WARNING\n"
"For -e, make sure you don't have W^X or any similar feature enabled.\n"
"\n"
"EXAMPLES\n"
"Execute shellcode:\t\tshellblade -e <object file>\n"
"Plain dump:\t\t\tshellblade -d <object file>\n"
"C-syntax shellcode:\t\tshellblade -c <object file>\n"
"ASM-syntax shellcode:\t\tshellblade -a <object file>\n"
"\n"
"DETAILS\n"
"The object file shellblade expects is a raw object file, stripped of any symbols.\n"
"To produce such a file using nasm, assemble with the following command:\n\n"
"nasm source.s\n\n"
"Notice we have not passed -f elf or any file format.\n"
"The resulting file in this case would be source.o. This is the file shellblade expects.\n\n"
"I like to give this file a different extension, to distinguish it from regular object files:\n\n"
"nasm source.s -o source.code\n"
"\n";

static void usage(const char *argvzero)
{
	printf("ShellBlade version %s\n\n", VERSION);
	printf("%s", help);
	
	fflush(stderr);
	exit(1);
}

void error(const char* str)
{
	perror(str);
	exit(-1);
}

int main(int argc, char **argv)
{
	FILE	*fp;
	long	len;
	struct	stat sbuf;
	void	*shellcode;
	void	(*exec)(void);
	int		i = 0, arg, nullbytes = 0;
	
	
	if(argc != 3)
		usage(argv[0]);
	
	if(stat(argv[2], &sbuf))
		error("Stat() failed\n");
	
	len = (long)sbuf.st_size;
	
	if(!(shellcode = malloc(len)))
		error("Malloc() failed");
	
	if(!(fp = fopen(argv[2], "rb")))
		error("Fopen() failed");
	
	if(fread(shellcode, 1, len, fp) != len)
		error("Fread() failed");
	
	if(fclose(fp))
		error("FClose() failed");
	
	while((arg = getopt(argc, argv, "e:p:c:a:")) != -1)
	{
		switch(arg)
		{
			case 'e':
			{
				fprintf(stderr, "Executing shellcode...\n\n");
				fflush(stderr);
				
				exec = (void (*)(void))shellcode;
				(*exec)();
				
				break;
			}
				
			case 'p':
			{
				printf("\n Extracted shellcode (%d bytes):\n\n", len);
				
				nullbytes = 0;
				for(i = 0; i < len; ++i)
				{
					if(((char *) shellcode)[i] == 0)
					{
						printf("\x1b[1m");
						nullbytes++;
					}
					
					printf("\%02x", ((unsigned char *)shellcode)[i]);
					printf("\x1b[0m");
				}
				
				if(nullbytes)
					printf("\n\nWarning: %d nullbytes found", nullbytes);
				
				printf("\n\n");
				break;	
			}
				
			case 'c':
			{
				printf("\n/* Extracted shellcode (%d bytes): */\n", len);
				printf("\nchar shellcode[] =\n");
				
				nullbytes = 0;
				for(i = 0; i < len; ++i)
				{
					if(!(i%MAXBYTES))
					{
						if(i)
							printf("\"\n");
						
						printf("\t\"");
					}
									
					if(((char *) shellcode)[i] == 0)
					{
						printf("\x1b[1;31m");
						nullbytes++;
					}

					printf("\\x%02x", ((unsigned char *)shellcode)[i]);
					printf("\x1b[0m");
				}
				printf("\";\n");
				
				if(nullbytes)
					printf("\nWarning: %d nullbytes found\n", nullbytes);
				
				printf("\n");
				
				break;
			}
				
			case 'a':
			{
				printf("\n; Extracted shellcode (%d bytes):\n\n", len);
				
				nullbytes = 0;
				for(i = 0; i < len; ++i)
				{
					if(!(i%MAXBYTES))
					{
						if(i)
							printf("\n");
						
						printf("db ");
					}
					else
						printf(",");
					
					if(((char *) shellcode)[i] == 0)
					{
						printf("\x1b[1;31m");
						nullbytes++;
					}

					printf("0x%02x", ((unsigned char *)shellcode)[i]);
					printf("\x1b[0m");
				}
				
				if(nullbytes)
					printf("\n\nWarning: %d nullbytes found", nullbytes);
				
				printf("\n\n");
				break;
			}
				
			default:
			{
				usage(argv[0]);
			}
		}
	}
	
	return 0;
}

